// Copyright 2022 ESRI
//
// All rights reserved under the copyright laws of the United States
// and applicable international laws, treaties, and conventions.
//
// You may freely redistribute and use this sample code, with or
// without modification, provided you include the original copyright
// notice and use restrictions.
//
// See the Sample code usage restrictions document for further information.
//

#include "Display_a_scene.h"


#include "ArcGISTiledElevationSource.h"

#include "Camera.h"

#include "ElevationSourceListModel.h"
#include "MapTypes.h"

#include "Scene.h"
#include "SceneQuickView.h"

#include "Surface.h"


#include <QUrl>

using namespace Esri::ArcGISRuntime;

Display_a_scene::Display_a_scene(QObject* parent /* = nullptr */):

    QObject(parent),

    m_scene(new Scene(BasemapStyle::ArcGISImagery, this))

{

    // create a new elevation source from Terrain3D rest service
    ArcGISTiledElevationSource* elevationSource = new ArcGISTiledElevationSource(
                QUrl("https://elevation3d.arcgis.com/arcgis/rest/services/WorldElevation3D/Terrain3D/ImageServer"), this);

    // add the elevation source to the scene to display elevation
    m_scene->baseSurface()->elevationSources()->append(elevationSource);



    setupScene();

}


Display_a_scene::~Display_a_scene()
{
}

SceneQuickView* Display_a_scene::sceneView() const
{
    return m_sceneView;
}

// Set the view (created in QML)
void Display_a_scene::setSceneView(SceneQuickView* sceneView)
{
    if (!sceneView || sceneView == m_sceneView)
    {
        return;
    }

    m_sceneView = sceneView;
    m_sceneView->setArcGISScene(m_scene);

    constexpr double latitude = 33.909;
    constexpr double longitude = -118.805;
    constexpr double altitude = 5330.0;
    constexpr double heading = 355.0;
    constexpr double pitch = 72.0;
    constexpr double roll = 0.0;
    const Camera sceneCamera(latitude, longitude, altitude, heading, pitch, roll);
    m_sceneView->setViewpointCameraAndWait(sceneCamera);

    emit sceneViewChanged();
}

void Display_a_scene::setupScene()
{

  ArcGISTiledElevationSource* elevationSource = new ArcGISTiledElevationSource(QUrl("https://elevation3d.arcgis.com/arcgis/rest/services/WorldElevation3D/Terrain3D/ImageServer"), this);

  Surface* elevationSurface = new Surface(this);
  elevationSurface->elevationSources()->append(elevationSource);

  elevationSurface->setElevationExaggeration(2.5);
  m_scene->setBaseSurface(elevationSurface);

}


